Type: Package
Package: smoothr
Title: Smooth and Tidy Spatial Features
Version: 1.3.0
Authors@R: 
    person(given = "Matthew",
           family = "Strimas-Mackey",
           role = c("aut", "cre"),
           email = "mstrimas@gmail.com",
           comment = c(ORCID = "0000-0001-8929-7776"))
Description: Tools for smoothing and tidying spatial features
    (i.e. lines and polygons) to make them more aesthetically pleasing.
    Smooth curves, fill holes, and remove small fragments from lines and
    polygons.
License: GPL-3
URL: https://strimas.com/smoothr/, https://github.com/mstrimas/smoothr
BugReports: https://github.com/mstrimas/smoothr/issues
Depends: R (>= 4.1.0)
Imports: sf, stats, units
Suggests: knitr, lwgeom, methods, rmarkdown, sp, testthat, terra
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-04 13:50:49 UTC; mes335
Author: Matthew Strimas-Mackey [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8929-7776>)
Maintainer: Matthew Strimas-Mackey <mstrimas@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-04 15:00:08 UTC
Built: R 4.4.3; ; 2026-05-04 21:44:14 UTC; unix
