# Generated by roxygen2: do not edit by hand

S3method(drop_crumbs,SpatVector)
S3method(drop_crumbs,Spatial)
S3method(drop_crumbs,sf)
S3method(drop_crumbs,sfc)
S3method(fill_holes,SpatVector)
S3method(fill_holes,Spatial)
S3method(fill_holes,sf)
S3method(fill_holes,sfc)
S3method(smooth,SpatVector)
S3method(smooth,Spatial)
S3method(smooth,sf)
S3method(smooth,sfc)
S3method(smooth,sfg)
export(densify)
export(drop_crumbs)
export(fill_holes)
export(smooth)
export(smooth_chaikin)
export(smooth_densify)
export(smooth_ksmooth)
export(smooth_spline)
