#' `smoothr`: Smooth and Tidy Spatial Features
#'
#' Tools for smoothing and tidying spatial features (i.e. lines and polygons) to
#' make them more aesthetically pleasing. Smooth curves, fill holes, and remove
#' small fragments from lines and polygons.
#'
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end
NULL
